#!/usr/bin/env bash

# fail asap
set -e

# Check if an argument was provided
if [ $# -eq 0 ]; then
    echo "No arguments provided"
    echo "Usage: scripts/publish-internal.sh <buildnum> [debug]"
    echo ""
    echo "First argument indicates the day's build number, starting from 0."
    echo "Last argument specifies whether we should have a debug build as opposed to release build."
    exit 1
fi

TAG=$(date -u +%Y%m%d)-$1
DEBUG=$2
if [ "$DEBUG" = "true" ]; then
  echo "[profile.release]" >> Cargo.toml
  echo "debug = true" >> Cargo.toml
  TAG=$TAG-debug
fi

echo "Building images"
docker build -t localhost:5000/revoltchat/base:latest -f Dockerfile.useCurrentArch .
docker build -t localhost:5000/revoltchat/server:$TAG - < crates/delta/Dockerfile
docker build -t localhost:5000/revoltchat/bonfire:$TAG - < crates/bonfire/Dockerfile
docker build -t localhost:5000/revoltchat/autumn:$TAG - < crates/services/autumn/Dockerfile
docker build -t localhost:5000/revoltchat/january:$TAG - < crates/services/january/Dockerfile
docker build -t localhost:5000/revoltchat/crond:$TAG - < crates/daemons/crond/Dockerfile
docker build -t localhost:5000/revoltchat/pushd:$TAG - < crates/daemons/pushd/Dockerfile

if [ "$DEBUG" = "true" ]; then
  git restore Cargo.toml
fi

echo "Pushing images to localhost:5000 with tag: $TAG!"
docker push localhost:5000/revoltchat/server:$TAG
docker push localhost:5000/revoltchat/bonfire:$TAG
docker push localhost:5000/revoltchat/autumn:$TAG
docker push localhost:5000/revoltchat/january:$TAG
docker push localhost:5000/revoltchat/crond:$TAG
docker push localhost:5000/revoltchat/pushd:$TAG

echo "Archiving source"
git archive --format=tar HEAD | bzip2 >../backend.tar.bz2
